//**********************************************************************;
// Copyright (c) 2015, CS2C Corporation
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
//
// 3. Neither the name of CS2C Corporation nor the names of its contributors
// may be used to endorse or promote products derived from this software without
// specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
// THE POSSIBILITY OF SUCH DAMAGE.
//**********************************************************************;

#ifndef _CS2C_TPM2_TOOL_H_
#define _CS2C_TPM2_TOOL_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <tss/TPM_Types.h>
#include <tss/tss_sysapi.h>
#include <tss/tss_fapi.h>
#include <tss/tpm20.h>
#include <tss/tss_error.h>
#include <tss/platform.h>
#include <tss/tss_defines.h>
#include <tss/tss_typedef.h>
#include <tss/tss_structs.h>
#include <tss/tspi.h>
#include <trousers/trousers.h>

/*operate result*/
#define 	OPER_INIT					-1
#define 	OPER_ERROR					-1
#define 	OPER_SUCC					0

/*error code*/
#define		RES_VER						0x0100
#define		RES_FMT						0x0080
#define		RES_WARN					0x0900
#define		RES_ERROR					0x0200
#define		RES_TAG						0x00C0
#define		RES_MASK					0x0040

/*hierarchy*/
#define     TPM2_RH_OWNER          (TPM_RH)(0x40000001)    
#define     TPM2_RH_NULL           (TPM_RH)(0x40000007)    
#define     TPM2_RH_ENDORSEMENT    (TPM_RH)(0x4000000B)    
#define     TPM2_RH_PLATFORM       (TPM_RH)(0x4000000C)
#define		TPM2_RS_PW			   (TPM_RH)(0x40000009)

/*algorithm support*/
/*yes:support, no: non-support*/
#define    TPM2_ALG_RSA               (TPM_ALG_ID)(0x0001)        //yes
#define    TPM2_ALG_DES               (TPM_ALG_ID)(0x0002)        //no
#define    TPM2_ALG__3DES             (TPM_ALG_ID)(0x0003)        //no
#define    TPM2_ALG_SHA               (TPM_ALG_ID)(0x0004)        //yes
#define    TPM2_ALG_SHA1              (TPM_ALG_ID)(0x0004)        //yes
#define    TPM2_ALG_HMAC              (TPM_ALG_ID)(0x0005)        //yes
#define    TPM2_ALG_AES               (TPM_ALG_ID)(0x0006)        //yes
#define    TPM2_ALG_MGF1              (TPM_ALG_ID)(0x0007)        //yes
#define    TPM2_ALG_KEYEDHASH         (TPM_ALG_ID)(0x0008)        //yes
#define    TPM2_ALG_XOR               (TPM_ALG_ID)(0x000A)        //yes
#define    TPM2_ALG_SHA256            (TPM_ALG_ID)(0x000B)        //yes
#define    TPM2_ALG_SHA384            (TPM_ALG_ID)(0x000C)        //no
#define    TPM2_ALG_SHA512            (TPM_ALG_ID)(0x000D)        //no
#define    TPM2_ALG_WHIRLPOOL512      (TPM_ALG_ID)(0x000E)        //no
#define    TPM2_ALG_NULL              (TPM_ALG_ID)(0x0010)        //yes
#define    TPM2_ALG_SM3_256           (TPM_ALG_ID)(0x0012)        //yes
#define    TPM2_ALG_SM4               (TPM_ALG_ID)(0x0013)        //yes
#define    TPM2_ALG_RSASSA            (TPM_ALG_ID)(0x0014)        //yes
#define    TPM2_ALG_RSAES             (TPM_ALG_ID)(0x0015)        //yes
#define    TPM2_ALG_RSAPSS            (TPM_ALG_ID)(0x0016)        //yes
#define    TPM2_ALG_OAEP              (TPM_ALG_ID)(0x0017)        //yes
#define    TPM2_ALG_ECDSA             (TPM_ALG_ID)(0x0018)        //yes
#define    TPM2_ALG_ECDH              (TPM_ALG_ID)(0x0019)        //yes
#define    TPM2_ALG_ECDAA             (TPM_ALG_ID)(0x001A)        //yes
#define    TPM2_ALG_SM2               (TPM_ALG_ID)(0x001B)        //yes
#define    TPM2_ALG_ECSCHNORR         (TPM_ALG_ID)(0x001C)        //yes
#define    TPM2_ALG_KDF1_SP800_56a    (TPM_ALG_ID)(0x0020)        //yes
#define    TPM2_ALG_KDF2              (TPM_ALG_ID)(0x0021)        //no
#define    TPM2_ALG_KDF1_SP800_108    (TPM_ALG_ID)(0x0022)        //yes
#define    TPM2_ALG_ECC               (TPM_ALG_ID)(0x0023)        //yes
#define    TPM2_ALG_SYMCIPHER         (TPM_ALG_ID)(0x0025)        //yes
#define    TPM2_ALG_CTR               (TPM_ALG_ID)(0x0040)        //yes
#define    TPM2_ALG_OFB               (TPM_ALG_ID)(0x0041)        //yes
#define    TPM2_ALG_CBC               (TPM_ALG_ID)(0x0042)        //yes
#define    TPM2_ALG_CFB               (TPM_ALG_ID)(0x0043)        //yes
#define    TPM2_ALG_ECB               (TPM_ALG_ID)(0x0044)        //yes

/*policy type*/
#define    TPM2_SE_HMAC      		  (TPM_SE)(0x00)    
#define    TPM2_SE_POLICY    		  (TPM_SE)(0x01)    
#define    TPM2_SE_TRIAL     		  (TPM_SE)(0x03)


/*Ecc curveID*/
#define    TPM2_ECC_NONE         	  (TPM_ECC_CURVE)(0x0000)    
#define    TPM2_ECC_NIST_P192    	  (TPM_ECC_CURVE)(0x0001)    
#define    TPM2_ECC_NIST_P224    	  (TPM_ECC_CURVE)(0x0002)    
#define    TPM2_ECC_NIST_P256    	  (TPM_ECC_CURVE)(0x0003)    
#define    TPM2_ECC_NIST_P384    	  (TPM_ECC_CURVE)(0x0004)    
#define    TPM2_ECC_NIST_P521    	  (TPM_ECC_CURVE)(0x0005)    
#define    TPM2_ECC_BN_P256      	  (TPM_ECC_CURVE)(0x0010)    
#define    TPM2_ECC_BN_P638      	  (TPM_ECC_CURVE)(0x0011)    
#define    TPM2_ECC_SM2_P256     	  (TPM_ECC_CURVE)(0x0020)


/*SessionData attributes*/
#define		TPM2_SESSION_CONTINUESESSION		0x00000001
#define		TPM2_SESSION_AUDITEXCLUSIVE			0x00000002
#define		TPM2_SESSION_AUDITRESET				0x00000004
#define		TPM2_SESSION_DECRYPT				0x00000008
#define		TPM2_SESSION_ENCRYPT				0x00000010
#define		TPM2_SESSION_AUDIT					0x00000020

/*Public attributes*/
#define		TPM2_PUBLIC_FIXEDTPM	    		0x00000001
#define		TPM2_PUBLIC_STCLEAR	    			0x00000002
#define		TPM2_PUBLIC_FIXEDPARENT	    		0x00000004
#define		TPM2_PUBLIC_SENSITIVEDATAORIGIN	    0x00000008
#define		TPM2_PUBLIC_USERWITHAUTH	    	0x00000010
#define		TPM2_PUBLIC_ADMINWITHPOLICY	    	0x00000020
#define		TPM2_PUBLIC_NODA	    			0x00000040
#define		TPM2_PUBLIC_ENCRYPTEDDUPLICATION	0x00000080
#define		TPM2_PUBLIC_RESTRICTED	    		0x00000100
#define		TPM2_PUBLIC_DECRYPT	    			0x00000200
#define		TPM2_PUBLIC_SIGN	    			0x00000400

/*NV attributes*/
#define 	TPM2_NV_PPWRITE						0x00000001
#define 	TPM2_NV_OWNERWRITE					0x00000002
#define 	TPM2_NV_AUTHWRITE   				0x00000004
#define 	TPM2_NV_POLICYWRITE 				0x00000008
#define 	TPM2_NV_COUNTER     				0x00000010
#define 	TPM2_NV_BITS        				0x00000020
#define 	TPM2_NV_EXTEND      				0x00000040
#define 	TPM2_NV_POLICY_DELETE  				0x00000080
#define 	TPM2_NV_WRITELOCKED    				0x00000100
#define 	TPM2_NV_WRITEALL       				0x00000200
#define 	TPM2_NV_WRITEDEFINE    				0x00000400
#define 	TPM2_NV_WRITE_STCLEAR  				0x00000800
#define 	TPM2_NV_GLOBALLOCK     				0x00001000
#define 	TPM2_NV_PPREAD         				0x00002000
#define 	TPM2_NV_OWNERREAD      				0x00004000
#define 	TPM2_NV_AUTHREAD       				0x00008000
#define 	TPM2_NV_POLICYREAD     				0x00010000
#define 	TPM2_NV_NO_DA          				0x00020000
#define 	TPM2_NV_ORDERLY        				0x00040000
#define 	TPM2_NV_CLEAR_STCLEAR  				0x00080000
#define 	TPM2_NV_READLOCKED     				0x00100000
#define 	TPM2_NV_WRITTEN        				0x00200000
#define 	TPM2_NV_PLATFORMCREATE 				0x00400000
#define 	TPM2_NV_READ_STCLEAR   				0x00800000

#define MAX_BYTE 1024

/*symcipher algorithm, keyBits and mode*/
typedef struct {
	UINT16 algorithm;
	UINT16 keyBits;
	UINT16 mode;
}Sym_Def_Object;

/*scheme*/
typedef struct {
	UINT16 scheme;
	UINT16 hashAlg;

	//count for TPM_ALG_ECDAA and TPM_ALG_ECSCHNORR
	UINT16 count;

	//kdf for keyedHash
	UINT16 kdf;
}Scheme_Detail;

/*unique*/
typedef struct {
	//unique for keyedHash,sym,rsa, or unique for ecc.x
	UINT16 unique_x_len;
	BYTE unique_x_data[MAX_BYTE];

	//unique for ecc.y
	UINT16 unique_y_len;
	BYTE unique_y_data[MAX_BYTE];
}Unique_Detail;

/*authPolicy*/
typedef struct {
	UINT16 public_authPolicy_len;
	BYTE   public_authPolicy_data[MAX_BYTE];
}AuthPolicy_Detail;

/*sensitive data*/
typedef struct {
	//sensitive size
	UINT16 size;

	//userAuth
	UINT16 userAuth_len;
	BYTE userAuth_data[MAX_BYTE];

	//userData
	UINT16 userData_len;
	BYTE userData_data[MAX_BYTE];
}Sensitive_Detail;

/*outside info*/
typedef struct {
	UINT16 outside_info_len;
	BYTE outside_info_data[MAX_BYTE];
}Outside_Info_Detail;

/*keyedHash params*/
typedef struct {
	Scheme_Detail keyedHash_scheme;
}KeyedHash_Parms;

/*symcipher params*/
typedef struct {
	Sym_Def_Object sym;
}Sym_Parms;

/*rsa params*/
typedef struct {
	Sym_Def_Object sym;
	Scheme_Detail rsa_scheme;
	UINT16 keyBits;
	UINT32 exponent;
}Rsa_Parms;

/*ecc params*/
typedef struct {
	Sym_Def_Object sym;
	Scheme_Detail ecc_scheme;
	UINT16 curveID;
	Scheme_Detail ecc_kdf_scheme;
}Ecc_Parms;

/*asym params*/
typedef struct {
	Sym_Def_Object sym;
	Scheme_Detail asym_scheme;
}Asym_Parms;

/*public params*/
typedef struct {
	UINT16 public_size;
	UINT16 public_type;
	UINT16 public_nameAlg;
	UINT32 public_attributes;
	AuthPolicy_Detail public_auth_policy_detail;
	KeyedHash_Parms public_keyedHash_parms;
	Sym_Parms public_sym_parms;
	Rsa_Parms public_rsa_parms;
	Ecc_Parms public_ecc_parms;
	Asym_Parms public_asym_parms;
	Unique_Detail public_unique_detail;
}Public_Detail;

/*sessionDataIn*/
typedef struct {
	//session number
	//BYTE numSessions;

	//session handle
	UINT32 sessionHandle;

	//nonce
	UINT16 nonce_len;
	BYTE nonce_data[MAX_BYTE];

	//session attributes
	BYTE sessionAttributes;

	//hmac
	UINT16 hmac_len;
	BYTE hmac_data[MAX_BYTE];
}Session_Data_In_Detail;

typedef struct {
	//xml path
	//char path[MAX_BYTE];
	
	//CreatePriamry---hierarchy, Create---primaryHandle
	UINT32 objectHandle;

	//session data in
	Session_Data_In_Detail session_data_in_detail;

	//sensitive
	Sensitive_Detail sensitive_detail;
	
	//public
	Public_Detail public_detail;

	//outside info
	Outside_Info_Detail outside_info_detail;

	//creation pcr
	TPML_PCR_SELECTION creationPcr;

	//independent scheme
	//Scheme_Detail scheme_detail;//for rsaDecrypt, rsaEncrypt, Sign, Quote, Certify, CertifyCreation, set scheme

	//independent sym_def_object
	//Sym_Def_Object sym_detail;//for Duplicate, Import, set sym_def_object
}Key_Detail;

/*creationHash*/
typedef struct {
	UINT16 len;
	BYTE data[MAX_BYTE];
}Buffer_Detail;

int
Tss2_Clear(
    TSS_HCONTEXT hContext,	/*in*/
    Key_Detail *key			/*in*/
);

int 
Tss2_ClearControl(
    TSS_HCONTEXT hConetxt,		/*in*/
    Key_Detail *key,			/*in*/
    BYTE disable				/*in*/
);

int 
Tss2_Certify(
	TSS_HCONTEXT hContext,					/*in*/
	Key_Detail *key,						/*in*/
	Key_Detail *sign_key,					/*in*/
	Buffer_Detail *qualifyingData_Detail,	/*in*/
	Scheme_Detail *scheme_detail,			/*in*/
	Buffer_Detail *certifyInfoData_Detail,	/*out*/
	TPMT_SIGNATURE *signature				/*out*/
);

int 
Tss2_CertifyCreation(
	TSS_HCONTEXT hContext,					/*in*/
	Key_Detail *sign_key,					/*in*/
	Key_Detail *key,						/*in(endorsement key)*/
	Buffer_Detail *qualifyingData_Detail,	/*in*/
	Buffer_Detail *creationHashData_Detail,	/*in(key's creationHash)*/
	Scheme_Detail *scheme_detail,			/*in*/
	TPMT_TK_CREATION *creationTicket,		/*in(key's creationTicket)*/
	Buffer_Detail *certifyInfoData_Detail,	/*out*/
	TPMT_SIGNATURE *signature				/*out*/
);

int
Tss2_CreateKey(
	TSS_HCONTEXT hContext,						/*in*/
	Key_Detail *key,							/*in*/
	TPM2B_PRIVATE *outPrivate,					/*out*/
    TPM2B_PUBLIC *outPublic,					/*out*/
    TPM2B_CREATION_DATA *creationData,			/*out*/
    Buffer_Detail *creation_hash_detail,		/*out*/
	TPMT_TK_CREATION *creationTicket			/*out*/
);

int
Tss2_CreatePrimaryKey(
	TSS_HCONTEXT hContext,						/*in*/
	Key_Detail *key,							/*in*/
	UINT32 *objectHandle,						/*out*/
	Buffer_Detail *creation_hash_detail,		/*out*/
	Buffer_Detail *name_detail,					/*out*/
	TPM2B_PUBLIC *outPublic,					/*out*/
	TPM2B_CREATION_DATA *creationData,			/*out*/
	TPMT_TK_CREATION *creationTicket			/*out*/
);

int 
Tss2_LoadExternal(
	TSS_HCONTEXT hContext,				/*in*/
	TPM2B_SENSITIVE *pInPrivate,        /*in*/
    TPM2B_PUBLIC *pInPublic,            /*in*/
    UINT32 hierarchy,					/*in*/
    UINT32 *objectHandle,				/*out*/
    Buffer_Detail *nameData_Detail		/*out*/
);

int
Tss2_LoadKey(
	TSS_HCONTEXT hContext,				/*in*/
	Key_Detail *key,					/*in*/
	TPM2B_PRIVATE *outPrivate,			/*in*/
    TPM2B_PUBLIC *outPublic,			/*in*/
	UINT32 *objectHandle,				/*out*/
	Buffer_Detail *nameData_Detail		/*out*/
);

int 
Tss2_Unseal(
	TSS_HCONTEXT hContext,			/*in*/
	Key_Detail *key,				/*in*/
	Buffer_Detail *outData_Detail	/*out*/
);

int 
Tss2_ReadPublic(
	TSS_HCONTEXT hContext,						/*in*/
	UINT32 objectHandle,						/*in*/
	Buffer_Detail *nameData_Detail,				/*out*/
	Buffer_Detail *qualifiedNameData_Detail,	/*out*/
	TPM2B_PUBLIC *outPublic						/*out*/
);

int 
Tss2_MakeCredential(
	TSS_HCONTEXT hContext,						/*in*/
	UINT32 objectHandle,						/*in*/
	Buffer_Detail *credentialData_Detail,		/*in*/
	Buffer_Detail *objectNameData_Detail,		/*in*/
	Buffer_Detail *credentialBlobData_Detail,	/*out*/
	Buffer_Detail *secretData_Detail			/*out*/
);

int 
Tss2_ActivateCredential(
	TSS_HCONTEXT hContext,					/*in*/
	Key_Detail *activate_key,				/*in*/
	Key_Detail *endorsement_key,			/*in*/
	Buffer_Detail *credentialBlob_Detail,	/*in*/
	Buffer_Detail *secretData_Detail,		/*in*/
	Buffer_Detail *certInfoData_Detail		/*out*/
);

int 
Tss2_Quote(
	TSS_HCONTEXT hContext,					/*in*/
	Key_Detail *key,						/*in*/
	Buffer_Detail *qualifyingData_Detail,	/*in*/
	Scheme_Detail *scheme_detail,			/*in*/
	TPML_PCR_SELECTION *pcrSelect,			/*in*/
	Buffer_Detail *quotedData_Detail,		/*out*/
	TPMT_SIGNATURE *signature				/*out*/
);

int 
Tss2_GetCapability(
	TSS_HCONTEXT hContext,					/*in*/
	UINT32 capability,						/*in*/
	UINT32 property,                        /*in*/
    UINT32 propertyCount,					/*in*/
	BYTE *moreData,							/*out*/
	TPMS_CAPABILITY_DATA *capabilityData	/*out*/	
);

int 
Tss2_GetRandom(
	TSS_HCONTEXT hContext,				/*in*/
	int numBytes,						/*in*/
	BYTE *outData						/*out*/
);

int 
Tss2_FlushContext(
	TSS_HCONTEXT hContext,			/*in*/
    UINT32 objectHandle				/*in*/
);

int 
Tss2_EvictControl(
	TSS_HCONTEXT hContext,				/*in*/
	Key_Detail *key,					/*in*/
	UINT32 objectHandle,				/*in*/
	UINT32 persistentHandle				/*in*/
);

int
Tss2_HashData(
	TSS_HCONTEXT hContext,				/*in*/
	Buffer_Detail*inData_Detail,		/*in*/
	UINT16 hashAlg,						/*in*/
	UINT32 hierarchy,					/*in*/
	Buffer_Detail *outHashData_Detail,	/*out*/
	TPMT_TK_HASHCHECK *validation		/*out*/
);

int 
Tss2_HashSequence(
	TSS_HCONTEXT hContext,						/*in*/
	Key_Detail *hierarchy_key,					/*in*/
	Buffer_Detail *inData_Detail,				/*in*/
	UINT16 hashAlg,								/*in*/
	UINT32 hierarchy,							/*in*/
	Buffer_Detail *outHashData_Detail,			/*out*/
	TPMT_TK_HASHCHECK *validation				/*out*/
);

int
Tss2_HierarchyControl(
	TSS_HCONTEXT hContext,			/*in*/
	Key_Detail *hierarchy_key,		/*in*/
	BYTE state						/*in*/
);

int
Tss2_HierarchyChangeAuth(
	TSS_HCONTEXT hContext,				/*in*/
	Key_Detail*hierarchy_key,			/*in*/
	Buffer_Detail *newAuthData_Detail	/*in*/
);

int 
Tss2_ObjectChangeAuth(
	TSS_HCONTEXT hContext,						/*in*/
	Key_Detail *key,							/*in*/
	UINT32 parentHandle,						/*in*/
	Buffer_Detail *objectNewAuthData_Detail,	/*in*/
	TPM2B_PRIVATE *outPrivate					/*out*/
);

int
Tss2_NV_DefineSpace(
	TSS_HCONTEXT hContext,			/*in*/ 
	Key_Detail *hierarchy_key,		/*in*/
	Buffer_Detail *authData_Detail,	/*in*/
	UINT32 nvAttributes,			/*in*/
	UINT32 nvIndex,					/*in*/
	UINT16 hashAlg,					/*in*/
	UINT16 nvDataSize				/*in*/
);

int 
Tss2_NV_Write(
	TSS_HCONTEXT hContext,				/*in*/
	Key_Detail *nv_key,					/*in*/
	UINT32 nvIndex, 					/*in*/
	UINT16 offset,						/*in*/
	Buffer_Detail *nvWriteData_Detail	/*in*/
);

int 
Tss2_NV_Read(
	TSS_HCONTEXT hContext,				/*in*/
	Key_Detail *nv_key,					/*in*/
	UINT32 nvIndex, 					/*in*/
	UINT16 size,						/*in*/
	UINT16 offset,						/*in*/
	Buffer_Detail *nvReadData_Detail	/*out*/
);

int 
Tss2_NV_UndefineSpace(
	TSS_HCONTEXT hContext,
	Key_Detail *hierarchy_key,
	UINT32 nvIndex
);

int
Tss2_Init_PcrSelection(
    TPML_PCR_SELECTION *pcrSelectionIn  /*in, out*/
);

int
Tss2_Set_PcrSelection_Data(
    TPML_PCR_SELECTION *pcrSelectionIn, /*in, out*/
	UINT16 hashAlg,                     /*in*/
	UINT32 pcrIndex                     /*in*/
);

int
Tss2_Get_ListDigest_Count(
    TPML_DIGEST *pcrValues,                     /*in*/
	UINT16 *count                               /*out*/
);

int
Tss2_Get_ListDigest_Data(
	TPML_DIGEST *pcrValues,						/*in*/
	UINT16 index,								/*in*/
	Buffer_Detail *pcrData_Detail				/*out*/
);

int
Tss2_Init_ListDigestValues(
    TPML_DIGEST_VALUES *digests                 /*in, out*/
);

int
Tss2_Set_ListDigestValues_Data(
	TPML_DIGEST_VALUES *digests,				/*in, out*/
	UINT16 hashAlg,								/*in*/
	Buffer_Detail *extendData_Detail			/*in*/
);

int
Tss2_Get_ListDigestValues_Count(
    TPML_DIGEST_VALUES *digests,                /*in*/
	UINT16 *count                               /*out*/
);

int
Tss2_Get_ListDigestValues_Data(
    TPML_DIGEST_VALUES *digests,                /*in*/
	UINT16 index,                               /*in*/
	UINT16 *hashAlg,                            /*out*/
	BYTE *eventData                             /*out*/
);

int 
Tss2_PCR_Event(
	TSS_HCONTEXT hContext,				/*in*/
	Key_Detail *pcr_key,				/*in*/
	Buffer_Detail *eventData_Detail,	/*in*/
	TPML_DIGEST_VALUES *digests			/*out*/
);

int 
Tss2_PCR_Read(
	TSS_HCONTEXT hContext,				/*in*/
	TPML_PCR_SELECTION *pcrSelectionIn,	/*in*/
	UINT32 *pcrUpdateCounter,			/*out*/
	TPML_DIGEST *pcrValues,				/*out*/
	TPML_PCR_SELECTION *pcrSelectionOut	/*out*/
);

int 
Tss2_PCR_Extend(
	TSS_HCONTEXT hContext,				/*in*/
	Key_Detail *pcr_key,				/*in*/
	TPML_DIGEST_VALUES *digests			/*in*/
);

int 
Tss2_PCR_Reset(
	TSS_HCONTEXT hContext,		/*in*/
	Key_Detail *pcr_key			/*in*/
);

int 
Tss2_PCR_SetAuthValue(
	TSS_HCONTEXT hContext,					/*in*/
	Key_Detail *pcr_key,					/*in*/
	Buffer_Detail *pcrNewAuthData_Detail	/*in*/
);

int 
Tss2_SymEncrypt(
	TSS_HCONTEXT hContext,				/*in*/ 
	Key_Detail *key,					/*in*/
	UINT16 mode,						/*in*/
	Buffer_Detail *ivData_Detail,		/*in*/
	Buffer_Detail *inData_Detail,		/*in*/
	Buffer_Detail *outData_Detail,		/*out*/
	Buffer_Detail *ivOutData_Detail 	/*out*/
);

int 
Tss2_SymDecrypt(
	TSS_HCONTEXT hContext,				/*in*/ 
	Key_Detail *key,					/*in*/
	UINT16 mode,						/*in*/
	Buffer_Detail *ivData_Detail,		/*in*/
	Buffer_Detail *inData_Detail,		/*in*/
	Buffer_Detail *outData_Detail,		/*out*/
	Buffer_Detail *ivOutData_Detail 	/*out*/
);

int 
Tss2_AsymEncrypt(
	TSS_HCONTEXT hContext,				/*in*/ 
	Key_Detail *rsa_encrypt_key,		/*in*/
	Buffer_Detail *labelData_Detail,	/*in*/
	Scheme_Detail *scheme_detail,		/*in*/
	Buffer_Detail *inData_Detail,		/*in*/
	Buffer_Detail *outData_Detail		/*out*/
);

int 
Tss2_AsymDecrypt(
	TSS_HCONTEXT hContext,				/*in*/ 
	Key_Detail *rsa_decrypt_key,		/*in*/
	Buffer_Detail *labelData_Detail,	/*in*/
	Scheme_Detail *scheme_detail,		/*in*/
	Buffer_Detail *inData_Detail,		/*in*/
	Buffer_Detail *outData_Detail		/*out*/
);

int 
Tss2_Sign(
	TSS_HCONTEXT hContext,				/*in*/
	Key_Detail *sign_key,				/*in*/
	Buffer_Detail *hashData_Detail,		/*in*/
	Scheme_Detail *scheme_detail,		/*in*/
	TPMT_TK_HASHCHECK *validation,		/*in*/ 
	TPMT_SIGNATURE *signature			/*out*/
);

int 
Tss2_VerifySignature(
	TSS_HCONTEXT hContext,				/*in*/ 
	UINT32 signKeyHandle,				/*in*/
	Buffer_Detail *hashData_Detail,		/*in*/
	TPMT_SIGNATURE *signature,			/*in*/
	TPMT_TK_VERIFIED *validation		/*out*/
);

int
Tss2_Get_Verified_Data(
	TPMT_TK_VERIFIED *validation,		/*in*/
	UINT16 *tag,						/*out*/
	UINT32 *hierarchy,					/*out*/
	Buffer_Detail *verifiedData_Detail	/*out*/
);

int
Tss2_Get_Signature_Alg(
	TPMT_SIGNATURE *signature,          /*in*/
	UINT16 *sigAlg                      /*out*/
);

int
Tss2_Get_Signature_Alg_Rsa(
	TPMT_SIGNATURE *signature,          /*in*/
	UINT16 sigAlg,						/*in*/
	UINT16 *hashAlg,					/*out*/
	Buffer_Detail *sigData_Detail		/*out*/
);

int
Tss2_Get_Signature_Alg_Ecc(
	TPMT_SIGNATURE *signature,          /*in*/
	UINT16 sigAlg,						/*in*/
	UINT16 *hashAlg,					/*out*/
	Buffer_Detail *sigR_Data_Detail,	/*out*/
	Buffer_Detail *sigS_Data_Detail		/*out*/
);

int
Tss2_Get_Signature_Alg_Hmac(
    TPMT_SIGNATURE *signature,          /*in*/
	UINT16 *hashAlg,                    /*out*/
	BYTE *sigData                       /*out*/
);

int
Tss2_Get_Signature_Alg_Any(
    TPMT_SIGNATURE *signature,          /*in*/
	UINT16 *hashAlg                     /*out*/
);

int 
Tss2_Key_Duplicate(
	TSS_HCONTEXT hContext,					/*in*/
	Key_Detail *key,						/*in*/
	UINT32 newParentHandle,					/*in*/
	Buffer_Detail *encryptionKeyIn_Detail,	/*in*/
	Sym_Def_Object *sym_detail,				/*in*/
	Buffer_Detail *encryptionKeyOut_Detail,	/*out*/
	TPM2B_PRIVATE *duplicate,				/*out*/
	Buffer_Detail *outSymSeed_Detail		/*out*/
);

int 
Tss2_Key_Import(
	TSS_HCONTEXT hContext,					/*in*/
	Key_Detail *key,						/*in*/
	Buffer_Detail *encryptionKey_Detail,	/*in*/
	TPM2B_PUBLIC *objectPublic,				/*in*/
	TPM2B_PRIVATE *duplicate,				/*in*/
	Buffer_Detail *inSymSeed_Detail,		/*in*/
	Sym_Def_Object *sym_detail,				/*in*/
	TPM2B_PRIVATE *outPrivate				/*out*/
);

int
Tss2_Policy_StartAuthSession(
	TSS_HCONTEXT hContext,				/*in*/
	UINT32 tpmKey,						/*in*/
	UINT32 bind,						/*in*/
	Buffer_Detail *inNonce_detail,		/*in*/
	Buffer_Detail *secret_detail,		/*in*/
	BYTE sessionType,					/*in*/
	Sym_Def_Object *sym_detail,			/*in*/
	UINT16 authHash,					/*in*/
	UINT32 *sessionHandle,				/*out*/
	Buffer_Detail *outNonce_detail		/*out*/
);

int
Tss2_Policy_CommandCode(
	TSS_HCONTEXT hContext,				/*in*/
	UINT32 sessionHandle,				/*in*/
	UINT32 code							/*in*/
);

int
Tss2_Policy_GetDigest(
	TSS_HCONTEXT hContext,				/*in*/
	UINT32 sessionHandle,				/*in*/
	Buffer_Detail *policyHash_detail	/*out*/
);

void
Tss2_ErrCode(
	int err_code,						/*in*/
	int *err_type,						/*out*/
	int *err_offset						/*out*/
);
#endif
